import { JSONExt } from '@lumino/coreutils';
import { CodeSnippetWidget } from './CodeSnippetWidget';
export class CodeSnippetService {
    constructor(settings, app) {
        this.settingManager = settings;
        // when user changes the snippets using settingsEditor
        this.settingManager.changed.connect(async (plugin) => {
            const newCodeSnippetList = plugin.get('snippets').user;
            if (!JSONExt.deepEqual(newCodeSnippetList, this.codeSnippetList)) {
                this.codeSnippetList = this.convertToICodeSnippetList(newCodeSnippetList);
                const leftWidgets = app.shell.widgets('left').iter();
                let current = leftWidgets.next();
                while (current) {
                    if (current instanceof CodeSnippetWidget) {
                        current.updateCodeSnippetWidget();
                        break;
                    }
                    current = leftWidgets.next();
                }
                // order code snippet and sync it with setting
                this.orderSnippets();
                await plugin
                    .set('snippets', this.codeSnippetList)
                    .catch((e) => {
                    console.log('Error in syncing orders of snippets with those in settings');
                    console.log(e);
                });
            }
        });
        // load user's saved snippets
        if (this.settingManager.get('snippets').user) {
            const userSnippets = this.convertToICodeSnippetList(this.settingManager.get('snippets').user);
            this.codeSnippetList = userSnippets;
        }
        // set default preview font size
        if (this.settingManager.get('snippetPreviewFontSize').user === undefined) {
            this.settingManager.set('snippetPreviewFontSize', this.settingManager.default('snippetPreviewFontSize'));
        }
    }
    convertToICodeSnippetList(snippets) {
        const snippetList = [];
        snippets.forEach((snippet) => {
            const newSnippet = snippet;
            snippetList.push(newSnippet);
        });
        return snippetList;
    }
    static init(settings, app) {
        if (!this.codeSnippetService) {
            this.codeSnippetService = new CodeSnippetService(settings, app);
        }
    }
    static getCodeSnippetService() {
        return this.codeSnippetService;
    }
    get settings() {
        return this.settingManager;
    }
    get snippets() {
        return this.codeSnippetList;
    }
    getSnippetByName(snippetName) {
        return this.codeSnippetList.filter((snippet) => snippet.name.toLowerCase() === snippetName.toLowerCase());
    }
    async addSnippet(snippet) {
        const id = snippet.id;
        this.codeSnippetList.splice(id, 0, snippet);
        const numSnippets = this.codeSnippetList.length;
        // update id's of snippets.
        let i = id + 1;
        for (; i < numSnippets; i++) {
            this.codeSnippetList[i].id += 1;
        }
        await this.settingManager
            .set('snippets', this.codeSnippetList)
            .catch((_) => {
            return false;
        });
        return true;
    }
    async deleteSnippet(id) {
        let numSnippets = this.codeSnippetList.length;
        // should never satisfy this condition
        if (id >= numSnippets) {
            console.log('error in codeSnippetService');
        }
        if (id === numSnippets - 1) {
            this.codeSnippetList.pop();
        }
        else {
            this.codeSnippetList.splice(id, 1);
            numSnippets = this.codeSnippetList.length;
            let i = id;
            for (; i < numSnippets; i++) {
                this.codeSnippetList[i].id -= 1;
            }
        }
        await this.settingManager
            .set('snippets', this.codeSnippetList)
            .catch((_) => {
            return false;
        });
        return true;
    }
    async renameSnippet(oldName, newName) {
        for (const snippet of this.codeSnippetList) {
            if (snippet.name === oldName) {
                snippet.name = newName;
                break;
            }
        }
        await this.settingManager
            .set('snippets', this.codeSnippetList)
            .catch((_) => {
            return false;
        });
        return true;
    }
    duplicateNameExists(newName) {
        for (const snippet of this.codeSnippetList) {
            if (snippet.name.toLowerCase() === newName.toLowerCase()) {
                return true;
            }
        }
        return false;
    }
    async modifyExistingSnippet(oldName, newSnippet) {
        for (const snippet of this.codeSnippetList) {
            if (snippet.name.toLowerCase() === oldName.toLowerCase()) {
                this.codeSnippetList.splice(snippet.id, 1, newSnippet);
                break;
            }
        }
        await this.settingManager
            .set('snippets', this.codeSnippetList)
            .catch((_) => {
            return false;
        });
        return true;
    }
    async moveSnippet(fromIdx, toIdx) {
        if (toIdx > fromIdx) {
            toIdx = toIdx - 1;
        }
        if (toIdx === fromIdx) {
            return;
        }
        const snippetToMove = this.codeSnippetList[fromIdx];
        this.deleteSnippet(fromIdx).then((res) => {
            if (!res) {
                console.log('Error in moving snippet(delete)');
                return false;
            }
        });
        snippetToMove.id = toIdx;
        this.addSnippet(snippetToMove).then((res) => {
            if (!res) {
                console.log('Error in moving snippet(add)');
                return false;
            }
        });
        await this.settingManager
            .set('snippets', this.codeSnippetList)
            .catch((_) => {
            return false;
        });
        return true;
    }
    // order snippets just in case when it gets shared between users
    async orderSnippets() {
        this.codeSnippetList.sort((a, b) => a.id - b.id);
        this.codeSnippetList.forEach((snippet, i) => (snippet.id = i));
        await this.settingManager
            .set('snippets', this.codeSnippetList)
            .catch((_) => {
            return false;
        });
        return true;
    }
}
